@@ -11,6 +11,14 @@ import os
 import sys
 import tornado.web
 
+
+def __get_webapp_root_dir():
+    """
+    Absolute path to webapp static resources
+    """
+    relative_path = '/webapp/'
+    return os.path.dirname(__file__) + relative_path
+
 def __error_handler(handler):
     """
     A Tornado handler that does nothing and returns {},
@@ -144,7 +152,7 @@ def __get_static_handlers():
     """
     Get a list of handlers for static files as tornado.web.RequestHandler instances.
     """
-    return [("/(.*\\.%s)" % ext, static.Handler, {'path': os.path.dirname(__file__)}) for ext in \
+    return [("/(.*\\.%s)" % ext, static.Handler, {'path': __get_webapp_root_dir()}) for ext in \
       ['css',
        'html',
        'ico',
@@ -172,8 +180,7 @@ def get_handlers(server, state):
         __make_handler_class(
             url=r"/metrics",
             post=__make_handler(__post_metrics, state)),
-        (r"/(index\.html)", IndexHandler, {'path': os.path.dirname(__file__)}),
-        (r"/(js\/.*)", static.Handler, {'path': os.path.dirname(__file__)}),
+        (r"/(index\.html)", IndexHandler, {'path': __get_webapp_root_dir()})
     ]
     ret.extend(__get_static_handlers())
     ret.extend([
