@@ -41,7 +41,7 @@ class GraphLabServer(object):
         try:
             self.stop()
         except:
-            e = sys.exc_info()[0]            
+            e = sys.exc_info()[0]
             self.get_logger().warning(e)
 
     def get_logger(self):
@@ -55,7 +55,7 @@ class LocalServer(GraphLabServer):
     unity_server process and implements the GraphLabServer interface.
     """
 
-    __slot__ = ["server_bin", "server_addr", "unity_log", "proc", "log_file_handle", "product_key", "auth_token"]
+    __slot__ = ["server_bin", "server_addr", "unity_log", "proc", "product_key", "auth_token"]
 
     def __init__(self, server_addr, server_bin, unity_log_file, product_key,
                  auth_token=None, public_key='', secret_key=''):
@@ -79,7 +79,6 @@ class LocalServer(GraphLabServer):
         self.auth_token = auth_token
         self.logger = logging.getLogger(__name__)
         self.proc = None
-        self.log_file_handle = None
         self.public_key = public_key
         self.secret_key = secret_key
 
@@ -113,15 +112,9 @@ class LocalServer(GraphLabServer):
             raise ValueError('Invalid server binary \"%s\" does not exist.' % self.server_bin)
         if not os.access(self.server_bin, os.X_OK):
             raise ValueError('Invalid server binary \"%s\" is not executable. Please check your file permission.' % self.server_bin)
-        try:
-            self.log_file_handle = open(self.unity_log, 'wb')
-        except IOError:
-            self.logger.warning("Unable to open log file \"%s\"." % self.unity_log)
-            self.log_file_handle = None
 
     def __del__(self):
         self.try_stop()
-        self.logger.info('GraphLab server shutdown')
 
     def get_server_addr(self):
         return self.server_addr
@@ -140,6 +133,10 @@ class LocalServer(GraphLabServer):
         if self.secret_key != '':
             arglist.append("--secret_key=%s" % self.secret_key)
 
+        arglist.append("--log_file=%s" % self.unity_log)
+        arglist.append("--log_rotation_interval=%d" % default_local_conf.log_rotation_interval)
+        arglist.append("--log_rotation_truncate=%d" % default_local_conf.log_rotation_truncate)
+
         env = os.environ.copy()
         classpath = _get_hadoop_class_path()
         if ("CLASSPATH" in env):
@@ -148,15 +145,14 @@ class LocalServer(GraphLabServer):
             env["CLASSPATH"] = classpath
 
         libjvm_path = _get_libjvm_path()
-        if ('LD_LIBRARY_PATH' in env):
-            env['LD_LIBRARY_PATH'] = env['LD_LIBRARY_PATH'] + (":" + libjvm_path if libjvm_path != '' else '')
+        libpython_path = _get_libpython_path()
+        if 'LD_LIBRARY_PATH' in env:
+            env['LD_LIBRARY_PATH'] = env['LD_LIBRARY_PATH'] + ":" + libpython_path + (":" + libjvm_path if libjvm_path != '' else '')
         else:
-            env['LD_LIBRARY_PATH'] = libjvm_path
+            env['LD_LIBRARY_PATH'] = libpython_path + ":" + libjvm_path
 
-        if ('PYTHONPATH' in env):
-            env['PYTHONPATH'] = env['PYTHONPATH'] + ":."
-        else:
-            env['PYTHONPATH'] = "." 
+        # Save the entire syspath, to environemnt
+        env['__GL_SYS_PATH__'] = ':'.join(sys.path)
 
         # Start a local server as a child process.
         if self.server_addr == 'default':
@@ -166,9 +162,10 @@ class LocalServer(GraphLabServer):
             if (protocol is "ipc" and os.path.exists(self.server_addr[6:])):
                 raise RuntimeError('Cannot start local server: local address %s already exists' % self.server_addr)
         try:
+            FNULL = open(os.devnull, 'w')
             self.proc = subprocess.Popen(arglist,
                                          env=env,
-                                         stdin=subprocess.PIPE, stdout=self.log_file_handle,
+                                         stdin=subprocess.PIPE, stdout=FNULL,
                                          stderr=subprocess.STDOUT, bufsize=-1,
                                          preexec_fn=lambda: os.setpgrp())  # do not forward signal
         except OSError as e:
@@ -198,7 +195,7 @@ class LocalServer(GraphLabServer):
         retry = 0
         while retry < max_retry:
             try:
-                c = Client([], self.server_addr, num_tolerable_ping_failures, 
+                c = Client([], self.server_addr, num_tolerable_ping_failures,
                            public_key=client_public_key, secret_key=client_secret_key,
                            server_public_key=self.public_key)
                 if self.auth_token:
@@ -214,7 +211,6 @@ class LocalServer(GraphLabServer):
                 c.stop()
 
     def stop(self):
-        _get_metric_tracker().flush()
         num_polls_before_kill = 20
         if (self.proc):
             self.proc.communicate("\n")
@@ -239,9 +235,6 @@ class LocalServer(GraphLabServer):
                 except:
                     pass
             self.proc = None
-        if (self.log_file_handle):
-            self.log_file_handle.close()
-            self.log_file_handle = None
 
     def get_logger(self):
         return self.logger
@@ -299,7 +292,6 @@ class RemoteServer(GraphLabServer):
           c.stop()
 
     def stop(self):
-        _get_metric_tracker().flush()
         pass
 
     def get_logger(self):
@@ -340,9 +332,9 @@ def __get_expanded_classpath(classpath):
     if classpath is None or classpath == '':
         return ''
 
-    #  so this set comprehension takes paths that end with * to be globbed to find the jars, and then 
+    #  so this set comprehension takes paths that end with * to be globbed to find the jars, and then
     #  recombined back into a colon separated list of jar paths, removing dupes and using full file paths
-    jars = ':'.join({ ":".join([os.path.abspath(jarpath) for jarpath in _glob.glob(path + '.jar')]) 
+    jars = ':'.join({ ":".join([os.path.abspath(jarpath) for jarpath in _glob.glob(path + '.jar')])
         for path in classpath.split(':') if path.endswith('*')})
     logging.getLogger(__name__).debug('classpath being used: %s' % jars)
     return jars
@@ -357,14 +349,14 @@ def _get_libjvm_path():
     if cur_platform.startswith("macosx"):
         return ''
     else:
-        potential_paths = [ 
+        potential_paths = [
                 '/usr/lib/jvm/default-java/jre/lib/amd64/server',               # ubuntu / debian distros
                 '/usr/lib/jvm/java/jre/lib/amd64/server',                       # rhel6
                 '/usr/lib/jvm/jre/lib/amd64/server',                            # centos6
                 '/usr/lib64/jvm/jre/lib/amd64/server',                          # opensuse 13
                 '/usr/local/lib/jvm/default-java/jre/lib/amd64/server',         # alt ubuntu / debian distros
                 '/usr/local/lib/jvm/java/jre/lib/amd64/server',                 # alt rhel6
-                '/usr/local/lib/jvm/jre/lib/amd64/server',                      # alt centos6  
+                '/usr/local/lib/jvm/jre/lib/amd64/server',                      # alt centos6
                 '/usr/local/lib64/jvm/jre/lib/amd64/server',                    # alt opensuse 13
                 '/usr/local/lib/jvm/java-7-openjdk-amd64/jre/lib/amd64/server', # alt ubuntu / debian distros
                 '/usr/lib/jvm/java-7-openjdk-amd64/jre/lib/amd64/server',       # alt ubuntu / debian distros
@@ -377,4 +369,14 @@ def _get_libjvm_path():
                 return path
         return ''
 
-                
+
+def _get_libpython_path():
+    """
+    Return the path of libpython2.7.so in the current python environment.
+
+    The path is a guess based on the heuristics that most of the time the library
+    is under sys.exec_prefix + "/lib".
+    """
+    import sys
+    libpython_path = sys.exec_prefix + "/lib"
+    return libpython_path
