@@ -1,12 +1,9 @@
 """
 GraphLab offers multiple ways to work with your data beyond your desktop or
-laptop. GraphLab has a Data Pipeline framework for quickly building automated
-data products.
-
-The GraphLab Data Pipeline is a light-weight framework for building reusable
-units-of-work, specifying the execution environment (ex. EC2, Hadoop), with
-late-binding and loose coupling, and management tools to support asynchronous
-execution.
+laptop. GraphLab Create makes it easy to take predictive applications to
+production, enabling batch processing with GraphLab Data Pipelines, and
+real-time querying and consumption of Predictive Objects (ie. Models) with
+GraphLab Predictive Services.
 
 Explore the detailed functionality via the following API documentation or the
 `Deployment chapter of the User Guide
@@ -149,7 +146,7 @@ visualized in Canvas using this object. This object represents the portion of th
 workbench for Predictive Services that have been defined in GraphLab Create.
 
 Refer to :py:class:`~graphlab.deploy._session.ScopedSession` for how to manage
-predictictive object list.
+predictive object list.
 
 Examples
 --------
@@ -183,3 +180,5 @@ PredictiveService(s):
 >>> del graphlab.deploy.predictive_services[0]
 
 """)
+
+from _parallel import parallel_for_each
