@@ -652,7 +652,7 @@ class Task(object):
 
         See Also
         --------
-        graphlab.deploy.tasks.load
+        graphlab.deploy.tasks
         """
         self._session.save(self, typename=type(self).__name__)
         __LOGGER__.info("Task saved successfully.")
@@ -1080,9 +1080,6 @@ class Pipeline(object):
         self._session = _gl.deploy._default_session
         self.name = name
         self._modified_since_last_saved = None
-        # register with session
-        if not self._session.register(self):
-            __LOGGER__.warning("Unable to register this object with session.")
 
     def clone(self, name):
         """
@@ -1115,9 +1112,6 @@ class Pipeline(object):
 
         new.set_tasks(new_steps)
        
-        if session_aware is True:
-            self._session.register(new)
-
         return new
 
     def get_required_packages(self):
