@@ -191,13 +191,16 @@ class Job(object):
 
         step_dir = _os.path.join(tmp_dir, 'steps')
 
+        step_ids = [int(i) for i in _os.listdir(step_dir)]   # Save as ints so we can properly sort them.
+
         sequence = []
-        for step_idx in sorted(_os.listdir(step_dir)):
+        for step_idx in sorted(step_ids):
             cur_step_dir = _os.path.join(step_dir, str(step_idx))
             sequence.append([])
 
-            for task_idx in sorted(_os.listdir(cur_step_dir)):
-                cur_task_file_path = _os.path.join(cur_step_dir, task_idx)
+            task_ids = [int(i) for i in _os.listdir(cur_step_dir)]
+            for task_idx in sorted(task_ids):
+                cur_task_file_path = _os.path.join(cur_step_dir, str(task_idx))
 
                 cur_task_file = open(cur_task_file_path, 'r')
                 cur_task = _pickle.load(cur_task_file)
@@ -472,23 +475,47 @@ class LocalAsynchronousJob(Job):
 
 class Ec2Job(Job):
     def __init__(self, name, tasks, environment, ec2_dns_name, ec2_instance_id, s3_bucket,
-                 s3_state_key, s3_log_key, s3_status_key):
+                 s3_state_key, s3_log_dir, s3_status_key_prefix):
         super(Ec2Job, self).__init__(name, tasks=tasks, environment=environment)
 
         self.ec2_dns_name = ec2_dns_name
         self.ec2_instance_id = ec2_instance_id
         self.s3_bucket = s3_bucket
         self.s3_state_key = s3_state_key
-        self.s3_log_key = s3_log_key
-        self.s3_status_key = s3_status_key
+        self.s3_log_dir = s3_log_dir
+        self.s3_status_key_prefix = s3_status_key_prefix
         self.__s3_log_url_expire = None
         self.__s3_log_url = None
-        self.__s3_log_size = None
         self._start_time = None
         self._end_time = None
         self._final_status = None
 
+
+    def flush_logs(self):
+        '''
+        Flushes all logs associated with this job.
+        '''
+        status = self.get_status()
+        if status in ('Completed', 'Failed', 'Canceled'):
+            logging.info("The job is %s. Logs have already been flushed." % status)
+            return
+
+        url = "http://%s:9004/flush" % self.ec2_dns_name
+        try:
+            _requests.post(url)
+        except:
+            logging.error("Unable to flush logs. Please try again in a few minutes")
+
+
     def get_status(self):
+        '''
+        Returns status information about the job.
+
+        Returns
+        -------
+        String representation of status. One of the following strings:
+        'Pending', 'Running', 'Completed', 'Failed', 'Canceled', 'Unknown'
+        '''
         # First check if we've already know we've reached a final state/status.
         if self._final_status:
             return self._final_status
@@ -502,22 +529,49 @@ class Ec2Job(Job):
         bucket = s3_conn.get_bucket(self.s3_bucket, validate=False)
         k = bucket.get_key(self.s3_state_key)
 
-        # The key only gets deleted on success.
         if not k:
             self._get_task_status_from_s3()
-            self._final_status = 'Completed'
+            self._final_status = 'Unknown'
             self.save()
             return self._final_status
 
         status_from_s3 = k.read()
-        if status_from_s3 == 'Failed':
+        if status_from_s3 in ('Failed', 'Completed'):
             self._get_task_status_from_s3()
-            self._final_status = 'Failed'
+            self._final_status = status_from_s3
             self.save()
             return self._final_status
 
         return status_from_s3
 
+    def get_task_status_logs(self):
+        '''
+        Returns detailed information about task status. This information can also
+        be found by calling .show() on your job.
+        
+        Returns
+        -------
+        A dict with keys for each task that is either completed, running or failed.
+
+        See Also
+        --------
+        show
+
+        Examples
+        --------
+        >>> my_job.get_task_status_logs()
+        { 'my_task-0': 'Completed',
+          'my_task-0_runtime': '1.95039892197',
+          'my_task-0_starttime': '2014-12-18 18:52:40.811694',
+          'my_task-1': 'Failed',
+          'my_task-1_starttime': '2014-12-18 19:05:29.037241',
+          'my_task-2': 'Running',
+          'my_task-2_starttime': '2014-12-18 18:52:42.763024'
+        }
+        '''
+        return self._get_task_status_from_s3()
+        
+
     def _get_start_time(self):
         if self._start_time:
             return str(self._start_time.isoformat())
@@ -547,10 +601,12 @@ class Ec2Job(Job):
             credentials['aws_secret_access_key'] = self.environment.aws_secret_key
         s3_conn = _connect_s3(**credentials)
         bucket = s3_conn.get_bucket(self.s3_bucket, validate=False)
-        k = bucket.get_key(self.s3_status_key)
+        if hasattr(self, 's3_status_key'): # Is this a job from prior to 1.2?
+            self.s3_status_key_prefix = self.s3_status_key
+        status_keys = bucket.get_all_keys(prefix = self.s3_status_key_prefix)
 
-        try:
-            if k:
+        for k in status_keys:
+            try:
                 jobstatus = k.read()
                 jobstatus_blob = jobstatus.split('\n')
                 for line in jobstatus_blob:
@@ -558,8 +614,8 @@ class Ec2Job(Job):
                     # spliting the line read, to key value pairs
                     if (len(line_split) > 1):
                         task_list[line_split[0]] = ":".join(line_split[1:]).rstrip('\n')
-        except IOError:
-            __LOGGER__.warning("Unable to get job status file from s3://%s/%s" % (self.s3_bucket, self.s3_status_key))
+            except IOError:
+                __LOGGER__.warning("Unable to get job status files from %s with prefix %s" % (self.s3_bucket, self.s3_status_key_prefix))
 
         self._task_status = task_list
         return task_list
@@ -598,7 +654,9 @@ class Ec2Job(Job):
         if self.__s3_log_url_expire and self.__s3_log_url:
             delta = _datetime.datetime.now() - self.__s3_log_url_expire
             if delta.total_seconds() < 300:
-                return (self.__s3_log_url, self.__s3_log_size)
+                return self.__s3_log_url
+
+        self.__s3_log_url_expire = _datetime.datetime.now()
 
         # Check s3 for the status key.
         credentials = {}   # If this is empty, boto will look in environment variables.
@@ -607,18 +665,28 @@ class Ec2Job(Job):
             credentials['aws_secret_access_key'] = self.environment.aws_secret_key
         s3_conn = _connect_s3(**credentials)
         bucket = s3_conn.get_bucket(self.s3_bucket, validate=False)
-        k = bucket.get_key(self.s3_log_key)
-
-        if not k:
-            return "Log file not available on S3."
+        if hasattr(self, 's3_log_key'): # Is this a job from prior to 1.2?
+            keys = [bucket.get_key(self.s3_log_key)]
         else:
+            keys = bucket.get_all_keys(prefix = self.s3_log_dir + 'i-')
+
+        if not keys:
+            return "Log file not available on S3."
+
+        results = []
+        for k in keys:
             url = k.generate_url(expires_in=300)
-            self.__s3_log_url = url
-            self.__s3_log_size = k.size
-            self.__s3_log_url_expire = _datetime.datetime.now()
-            return (url, k.size)
+            results.append((url, k.size))
+
+        self.__s3_log_url = results
+        return results
+
 
     def cancel(self):
+        """
+        Cancels this job. This will terminate all associated EC2 instances. Job status
+        will be marked as 'Canceled'.
+        """
         cancel_url = "http://%s:9004/cancel" % self.ec2_dns_name
         try:
             _requests.post(cancel_url)
@@ -626,17 +694,19 @@ class Ec2Job(Job):
             logging.error("Unable to cancel EC2 instance(s). Please check AWS Console to make sure "
                           + "any EC2 instances launched have been terminated.")
         else:
-            self._final_status = 'Cancel'
+            self._final_status = 'Canceled'
 
     def __str__(self):
         status = self.get_status()
         result = self._get_str()
         result += "Status: %s\n\n" % status
         result += "Execution Information:\n"
-        result += "  Public DNS: %s\n" % self.ec2_dns_name
-        result += "  Instance ID: %s\n" % self.ec2_instance_id
-        result += "  Status: %s\n" % self.get_status()
-        result += "  Log file: bucket = %s, key = %s\n" % (self.s3_bucket, self.s3_log_key)
+        result += "  Commander Public DNS: %s\n" % self.ec2_dns_name
+        result += "  Commander Instance ID: %s\n" % self.ec2_instance_id
+        if hasattr(self, 's3_log_key'): # Is this a job from prior to 1.2?
+            result += "  Log file: bucket = %s, key = %s\n" % (self.s3_bucket, self.s3_log_key)
+        else:
+            result += "  Log files: bucket = %s, directory = %s\n" % (self.s3_bucket, self.s3_log_dir)
         return result
 
 
