@@ -31,6 +31,7 @@ from _model_predictive_object import ModelPredictiveObject
 from _predictive_service_endpoint import PredictiveServiceEndpoint as _PredictiveServiceEndpoint
 from _predictive_object import PredictiveObject as _PredictiveObject
 from _custom_query_predictive_object import CustomQueryPredictiveObject as _CustomQueryPredictiveObject
+from _predictive_client import NonExistError as _NonExistError
 
 _logger = _getLogger(__name__)
 _name_checker = _compile('^[a-zA-Z-]+$')
@@ -168,6 +169,12 @@ class PredictiveService(object):
                 ret[po_name] = {'action': 'update', 'version':po_version}
         return ret
 
+    @staticmethod
+    def _get_bucket_region(s3_bucket_name, credentials):
+        conn = _connect_s3(**credentials)
+        bucket = conn.get_bucket(s3_bucket_name)
+        return bucket.get_location() or "us-east-1" # default=us-standard
+
     @staticmethod
     def _get_s3_state_config(s3_bucket_name, s3_key_name, credentials):
         conn = _connect_s3(**credentials)
@@ -251,7 +258,7 @@ class PredictiveService(object):
         result += 'HOSTS\n'
         for host in self._get_status():
             result += "%s (id: %s)\n" % (host['dns_name'], host['id'])
-            result +=  "State: %s" % host['state']
+            result += "State: %s" % host['state']
             if host['state'] != 'InService':
                 result += " - %s" % host['reason']
             result += "\nServing Models:\n"
@@ -368,7 +375,13 @@ class PredictiveService(object):
         '''
         self._ensure_not_terminated();
 
-        return self._environment.query(po_name, data, self._api_key)
+        try:
+            return self._environment.query(po_name, data, self._api_key)
+        except _NonExistError:
+            return "Predictive Object '%s' can not be found. If you just deployed "\
+                "the Predictive Object, it may take a short while for all Predictive " \
+                "Service nodes to be up-to-date. Please use get_predictive_objects_status() " \
+                "to get most current state." % po_name
 
 
     def add(self, name, obj, dependencies=None, required_packages=None, description=''):
@@ -847,38 +860,10 @@ class PredictiveService(object):
 
         _logger.info("retrieving metrics from predictive service...")
         try:
-            return self._environment.get_metrics_url()
+            return self._environment.get_metrics_url(self.name)
         except Exception as e:
             _logger.error("Error retrieving metrics: %" % e.message)
 
-    def _get_metrics(self, start_date=None, end_date=None):
-        '''
-        Get the metrics associated with the Predictive Service instance.
-
-        Parameters
-        -----------
-        start_date : datetime, optional
-            The begin time (in utc) to query metrics.
-
-        end_date : datetime, optional
-            The end time (in utc) to query metrics
-
-        Returns
-        -------
-        out : dict
-            Return a dictionary containing aggregated metric data as a time
-            series from the specified date range.
-
-        Notes
-        -----
-        If start_date and end_date are not given, metrics for the past 7 days
-        are retrieved.
-
-        '''
-        self._ensure_not_terminated();
-
-        return self._environment.get_metrics(start_date, end_date)
-
     def get_metrics(self, start_time=None, end_time=None, time_range=None, period=None):
         '''
         Get the metrics associated with the Predictive Service instance. The metrics includes:
@@ -914,7 +899,7 @@ class PredictiveService(object):
         --------
 
         # last 12 hours of metrics, with 5 minute (300 seconds) period
-        >>> ps._get_metrics()
+        >>> ps.get_metrics()
         ({'end_time': datetime.datetime(2014, 11, 13, 11, 36, 3, 157682),
           'start_time': datetime.datetime(2014, 11, 12, 23, 36, 3, 157682)},
         +------------------+---------------------------+-------------------+
@@ -928,7 +913,7 @@ class PredictiveService(object):
         ......
 
         # last 6 hours of metrics, with 1 minute (60 seconds) period
-        >>> ps._get_metrics(end_time = datetime.datetime.now(), time_range = 6, period = 60)
+        >>> ps.get_metrics(end_time = datetime.datetime.now(), time_range = 6, period = 60)
         ({'end_time': datetime.datetime(2014, 11, 13, 11, 49, 7, 743095),
           'start_time': datetime.datetime(2014, 11, 13, 5, 49, 7, 743095)},
         +------------------+---------------------------+-------------------+
@@ -1043,6 +1028,62 @@ class PredictiveService(object):
 
         self._session.save(self, typename="PredictiveService")
 
+    def cache_enable(self, name=None, restart=True):
+        """
+        Enable caching for a Predictive Service.
+
+        Parameters
+        ----------
+        name: str, optional
+            A unique identifier for the Predictive Object. If empty or None, the
+            cache will be enabled for the entire service.
+
+        restart: bool, optional (defaults to True)
+            A boolean indicating whether the underlying cache process should be
+            restarted (and all data erased).
+        """
+        self._ensure_not_terminated()
+        self._environment.cache_enable(name, restart)
+
+    def cache_disable(self, name=None):
+        """
+        Disable caching for a Predictive Service.
+
+        Parameters
+        ----------
+        name: str, optional
+            A unique identifier for the Predictive Object. If empty or None, the
+            cache will be disabled for the entire service.
+        """
+        self._ensure_not_terminated()
+        self._environment.cache_disable(name)
+
+    def cache_clear(self, name=None):
+        """
+        Clear a Predictive Service's cache.
+
+        Parameters
+        ----------
+        name: str, optional
+            A unique identifier for the Predictive Object. If a valid name is
+            specified, only cache entries associated with that particular
+            Predictive Object will be removed. Otherwise, if name is empty or
+            None, the entire cache is cleared.
+        """
+        self._ensure_not_terminated()
+        self._environment.cache_clear(name)
+
+    def flush_logs(self):
+        """
+        Force a Predictive Service to ship its logs to S3.
+
+        Logs are shipped periodically, but you may find it useful for diagnosing
+        unexpected behavior to ship the logs on demand. The logs will be shipped
+        to the location specified in the `EC2Environment` at the time the
+        Predictive Service was created.
+        """
+        self._ensure_not_terminated()
+        self._environment.flush_logs()
 
     def _save_state_to_s3(self):
         # Dump immutable state data to a config
