@@ -91,8 +91,8 @@ class Environment(object):
 
     def __getstate__(self):
         odict = dict.copy(self.__dict__)
-        if hasattr(odict, '_session'):
-            del odict._session
+        if '_session' in odict:
+            del odict['_session']
         return odict
 
     def __repr__(self):
@@ -175,6 +175,9 @@ class EC2(Environment):
         The AWS Secret Key to use to launch the host(s).  This parameter must be set or the
         AWS_SECRET_ACCESS_KEY environment variable must be set.
 
+    num_hosts : int, optional
+        The number of EC2 host(s) to use for this environment.
+
     region : str, optional
         The AWS region in which to launch your instance. Default is 'us-west-2'.
 
@@ -202,7 +205,7 @@ class EC2(Environment):
 
     """
     def __init__(self, name, s3_log_folder_path, aws_access_key = None, aws_secret_key = None,
-                 region = 'us-west-2', instance_type = _DEFAULT_INSTANCE_TYPE,
+                 num_hosts = 1, region = 'us-west-2', instance_type = _DEFAULT_INSTANCE_TYPE,
                  security_group = None, tags = None, CIDR_rule = None):
         super(EC2, self).__init__(name, _ENVIRONMENT.EC2)
 
@@ -220,6 +223,13 @@ class EC2(Environment):
                                        ' graphlab.aws.set_credentials(...).'
                                    )
 
+        try:
+            self.num_hosts = int(num_hosts)
+            if num_hosts <= 0:
+                raise TypeError("num_hosts must be a positive integer")
+        except ValueError:
+            raise TypeError("num_hosts is not an integer")
+
         if not isinstance(region, str):
             raise TypeError('region must be a string')
 
@@ -261,15 +271,13 @@ class EC2(Environment):
 
 
     def __str__(self):
-        return "EC2: [name: %s, access_key: %s, instance_type: %s, region: %s, params: %s, log folder: s3://%s/%s]" % \
+        return "EC2: [name: %s, access_key: %s, instance_type: %s, region: %s, num_hosts: %d, log folder: s3://%s/%s, params: %s]" % \
                (self.name, self.aws_access_key if hasattr(self, 'aws_access_key') else 'None',
-                self.instance_type, self.region, self.params, self.s3_bucket, self.s3_log_folder_path)
+                self.instance_type, self.region, self.num_hosts, self.s3_bucket, self.s3_log_folder_path, self.params)
 
     def get_max_degree_of_parallelism(self):
         """Return the number of tasks that can be executed in parallel."""
-        if hasattr(self, 'num_hosts'):
-            return self.num_hosts
-        return 1
+        return self.num_hosts
 
 
 class Hadoop(Environment):
@@ -287,6 +295,9 @@ class Hadoop(Environment):
     memory_mb : int
         The memory in MB required for job execution.  Default is 4096.
 
+    virtual_cores : int
+        The number of virtuals cores to use, must be at least two.
+
     gl_source : None | 'none' | HDFS full path | 'native'
         The location of a tar.gz containing GraphLab Create and all its requirements.
 
@@ -373,7 +384,7 @@ class Hadoop(Environment):
 
     def get_max_degree_of_parallelism(self):
         """Return the number of tasks that can be executed in parallel."""
-        return self.vcores
+        return 1
 
     def set_container_memory(self, memory_mb):
         """Set the memory (in MBs) this environment will request from Hadoop."""
