@@ -9,6 +9,7 @@ import os as _os
 import datetime as _datetime
 
 from boto.ec2.elb import connect_to_region as _lb_connection
+from boto.exception import S3ResponseError as _S3ResponseError
 from graphlab.connect.aws._ec2 import LicenseValidationException
 
 
@@ -139,6 +140,26 @@ def create(name, environment, state_path, description = None, api_key = None, ad
         'aws_secret_access_key': aws_secret_key
         }
 
+    # Warn if specified bucket is in different region than specified in env.
+    s3_bucket_name, _ = _parse_s3_path(state_path)
+
+    try:
+        region = _PredictiveService._get_bucket_region(s3_bucket_name,
+                                                       aws_credentials)
+    except _S3ResponseError as e:
+        _logger.error("Unable to connect to bucket; check your AWS credentials")
+        raise
+
+    if region != environment.region:
+        _logger.warn("The bucket in your state path is in a different region " \
+                     "(%s) than the one specified in your environment (%s). " \
+                     "AWS data transfer rates apply. Additionally, upload and " \
+                     "download speeds may be slower than expected. If this is " \
+                     "not what you intended, abort this operation or " \
+                     "terminate the service upon its completion, then be sure " \
+                     "that your environment and S3 bucket are in the same " \
+                     "region before relaunching." % (region, environment.region))
+
     # Validate 'name' value
     if not _name_checker.match(name):
         raise IOError('Names can only can only contain: a-z, A-Z and hyphens.')
