@@ -309,9 +309,9 @@ class BoostedTreesClassifier(_Classifier):
         if metric in ['confusion_matrix', 'auto']:
           predictions = self.predict(dataset, output_type = 'class')
           target = self.get('target')
-          _raise_error_if_column_exists(dataset, predictions, 'dataset', target + '(target column)')
+          _raise_error_if_column_exists(dataset, target, 'dataset', target)
           results['confusion_matrix'] = _graphlab.evaluation.confusion_matrix(\
-                                         predictions, dataset[target])
+                                         dataset[target], predictions)
         return results
 
     def predict(self, dataset, output_type='class'):
@@ -479,13 +479,14 @@ class BoostedTreesClassifier(_Classifier):
         '''Returns a list of method names that are queryable through Predictive
         Service'''
         return {'predict':{'dataset':'sframe'},
+                'predict_topk':{'dataset':'sframe'},
                 'classify':{'dataset':'sframe'}}
 
 
 def create(dataset, target,
            features=None, max_iterations=10,
            validation_set=None,
-           verbose=True,
+           verbose=True, class_weights = None,
            **kwargs):
     """
     Create a (binary or multi-class) classifier model of type
@@ -500,8 +501,8 @@ def create(dataset, target,
 
     target : str
         Name of the column containing the target variable. The values in this
-        column must of string or integer type.  String target variables are
-        autumatically mapped to integers in alphabetical order of the variable values.
+        column must be of string or integer type.  String target variables are
+        automatically mapped to integers in alphabetical order of the variable values.
         For example, a target variable with 'cat', 'dog', and 'foosa' as possible
         values is mapped to 0, 1, and, 2 respectively.
 
@@ -517,6 +518,14 @@ def create(dataset, target,
     max_depth : float, optional
         Maximum depth of a tree.
 
+    class_weights : {dict, `auto`}, optional
+
+        Weights the examples in the training data according to the given class
+        weights. If set to `None`, all classes are supposed to have weight one. The
+        `auto` mode set the class weight to be inversely proportional to number of
+        examples in the training data with the given class.
+
+
     step_size : float, [0,1], optional
         Step size (shrinkage) used in update to prevents overfitting.  It
         shrinks the prediction of each weak learner to make the boosting
@@ -599,5 +608,6 @@ def create(dataset, target,
                         model_name = 'boosted_trees_classifier',
                         max_iterations = max_iterations,
                         validation_set = validation_set,
+                        class_weights = class_weights,
                         verbose = verbose, **kwargs)
     return BoostedTreesClassifier(model.__proxy__)
