@@ -8,6 +8,9 @@ from graphlab.toolkits._supervised_learning import Classifier as _Classifier
 from graphlab.toolkits._internal_utils import _raise_error_evaluation_metric_is_valid
 from graphlab.toolkits._internal_utils import _toolkit_repr_print
 from graphlab.toolkits._model import _get_default_options_wrapper
+from graphlab.toolkits._internal_utils import _raise_error_if_not_sframe
+from graphlab.toolkits._internal_utils import _check_categorical_option_type
+from graphlab.toolkits._internal_utils import _map_unity_proxy_to_object
 
 _DEFAULT_SOLVER_OPTIONS = {
 'convergence_threshold': 1e-2,
@@ -51,8 +54,8 @@ def create(dataset, target, features=None, l2_penalty=0.01, l1_penalty=0.0,
 
     target : string
         Name of the column containing the target variable. The values in this
-        column must of string or integer type. String target variables are
-        autumatically mapped to integers in alphabetical order of the variable
+        column must be of string or integer type. String target variables are
+        automatically mapped to integers in alphabetical order of the variable
         values. For example, a target variable with 'cat' and 'dog' as possible
         values is mapped to 0 and 1 respectively with 0 being the base class 
         and 1 being the reference class. 
@@ -275,6 +278,7 @@ def create(dataset, target, features=None, l2_penalty=0.01, l1_penalty=0.0,
                         feature_rescaling = feature_rescaling,
                         convergence_threshold = convergence_threshold, 
                         step_size = step_size, 
+                        solver = solver, 
                         lbfgs_memory_level = lbfgs_memory_level, 
                         max_iterations = max_iterations, 
                         class_weights = class_weights)
@@ -381,6 +385,7 @@ class LogisticClassifier(_Classifier):
         model_fields = [
             ('Number of coefficients', 'num_coefficients'),
             ('Number of examples', 'num_examples'),
+            ('Number of classes', 'num_classes'),
             ('Number of feature columns', 'num_features'),
             ('Number of unpacked features', 'num_unpacked_features')]
 
@@ -684,6 +689,76 @@ class LogisticClassifier(_Classifier):
         return super(LogisticClassifier, self).classify(dataset,
                       missing_value_action = missing_value_action)
 
+    def predict_topk(self, dataset, output_type="probability", k=3):
+        """
+        Return top-k predictions for the ``dataset``, using the trained model.
+        Predictions are returned as an SFrame with three columns: `row_id`,
+        `class`, and `probability`, `margin`,  or `rank`, depending on the ``output_type``
+        parameter. Input dataset size must be the same as for training of the model.
+
+        Parameters
+        ----------
+        dataset : SFrame
+            A dataset that has the same columns that were used during training.
+            If the target column exists in ``dataset`` it will be ignored
+            while making predictions.
+
+        output_type : {'probability', 'rank', 'margin'}, optional
+            Choose the return type of the prediction:
+
+            - `probability`: Probability associated with each label in the prediction.
+            - `rank`       : Rank associated with each label in the prediction.
+            - `margin`     : Margin associated with each label in the prediction.
+
+        k : int, optional
+            Number of classes to return for each input example.
+
+        Returns
+        -------
+        out : SFrame
+            An SFrame with model predictions.
+
+        See Also
+        --------
+        predict, classify, evaluate
+
+        Examples
+        --------
+        >>> pred = m.predict_topk(validation_data, k=3)
+        >>> pred
+        +--------+-------+-------------------+                                           
+        | row_id | class |   probability     |
+        +--------+-------+-------------------+
+        |   0    |   4   |   0.995623886585  |
+        |   0    |   9   |  0.0038311756216  |
+        |   0    |   7   | 0.000301006948575 |
+        |   1    |   1   |   0.928708016872  |
+        |   1    |   3   |  0.0440889261663  |
+        |   1    |   2   |  0.0176190119237  |
+        |   2    |   3   |   0.996967732906  |
+        |   2    |   2   |  0.00151345680933 |
+        |   2    |   7   | 0.000637513934635 |
+        |   3    |   1   |   0.998070061207  |
+        |  ...   |  ...  |        ...        |
+        +--------+-------+-------------------+
+        [35688 rows x 3 columns]
+        """
+        _mt._get_metric_tracker().track(
+                      'toolkit.classifier.logistic_classifier.predict_topk')
+        _raise_error_if_not_sframe(dataset, "dataset")
+        _check_categorical_option_type('output_type', output_type, 
+                                          ['rank', 'margin', 'probability'])
+        options = dict()
+        options.update({'model': self.__proxy__,
+                        'model_name': self.__name__,
+                        'dataset': dataset,
+                        'output_type': output_type,
+                        'topk': k,
+                        'missing_value_action': 'error'})
+        target = _graphlab.toolkits._main.run(
+                          'supervised_learning_predict_topk', options)
+        return _map_unity_proxy_to_object(target['predicted'])
+
     def evaluate(self, dataset, metric = 'auto', missing_value_action = 'impute'):
         """
         Evaluate the model by making predictions of target values and comparing
@@ -787,6 +862,5 @@ class LogisticClassifier(_Classifier):
         '''Returns a list of method names that are queryable through Predictive
         Service'''
         return {'predict':{'dataset':'sframe'},
-                'classify':{'dataset':'sframe'}
-                }
-
+                'predict_topk':{'dataset':'sframe'},
+                'classify':{'dataset':'sframe'}}
