@@ -150,7 +150,7 @@ class NeuralNetClassifier(_Classifier):
         """
         Evaluate the model by making predictions of target values and comparing
         these to actual values. Input dataset must be the same size as for the
-        training of the model.
+        training of the model, except for images which are automatically resized.
 
         Parameters
         ----------
@@ -225,7 +225,8 @@ class NeuralNetClassifier(_Classifier):
     def predict(self, dataset, output_type="class"):
         """
         Return the model predictions for ``dataset``. Input dataset size must be
-        the same as for the training of the model.
+        the same as for the training of the model, except for images which are
+        automatically resized.
 
         Parameters
         ----------
@@ -274,7 +275,9 @@ class NeuralNetClassifier(_Classifier):
         of all the hidden unit values at layer[layer_id]. These feature vectors
         can be used as input to train another classifier such as a :py:class:`~graphlab.logistic_classifier.LogisticClassifier`,
         an :py:class:`~graphlab.svm_classifier.SVMClassifier`, another
-        :py:class:`~graphlab.neuralnet_classifier.NeuralNetClassifier`, or a :py:class:`~graphlab.boosted_trees_classifier.BoostedTreesClassifier`.
+        :py:class:`~graphlab.neuralnet_classifier.NeuralNetClassifier`, or a :py:class:`~graphlab.boosted_trees_classifier.BoostedTreesClassifier`. Input dataset size must be the same as for the training of the model,
+        except for images which are automatically resized.
+
 
         We also are releasing a pre-trained model for ImageNet, as described by
         Alex Krizhevsky et. al. It is located at
@@ -372,7 +375,8 @@ class NeuralNetClassifier(_Classifier):
         Return top-k predictions for the ``dataset``, using the trained model.
         Predictions are returned as an SFrame with three columns: `row_id`,
         `class`, and `score` or `rank`, depending on the ``output_type``
-        parameter. Input dataset size must be the same as for training of the model.
+        parameter. Input dataset size must be the same as for training of the
+        model, except for images which are automatically resized.
 
         Parameters
         ----------
@@ -446,7 +450,8 @@ class NeuralNetClassifier(_Classifier):
         Return a classification for each example in the ``dataset``, using the
         trained neural network model. The output SFrame contains predictions as
         both class labels and probabilities that the predicted class is correct.
-        Input dataset size must be the same as for the training of the model.
+        Input dataset size must be the same as for the training of the model,
+        except for images which are automatically resized.
 
         Parameters
         ----------
@@ -962,7 +967,6 @@ def create(dataset, target,
                        validation_set=validation_set,
                        verbose=verbose,
                        neuralnet_conf=_network.__config_str__(),
-                       cxxnet_layer_conf=_network._cxxnet_layer_str(),
                        _model_constructor=_cloudpickle.dumps(NeuralNetClassifier(None)._get_wrapper()),
                        # special handling for mean_image
                        # which be captured by neuralnet_conf
