@@ -8,8 +8,7 @@ import graphlab.connect as _mt
 import graphlab as _graphlab
 from graphlab.toolkits._model import Model as _Model
 from graphlab.data_structures.sframe import SFrame as _SFrame
-from graphlab.toolkits._internal_utils import _toolkit_repr_print
-from graphlab.util import validate_feature_types as _validate_feature_types
+from graphlab.toolkits._internal_utils import _toolkit_repr_print, _select_valid_features
 from array import array as _array
 import logging as _logging
 import json as _json
@@ -26,8 +25,10 @@ class KmeansModel(_Model):
     on a dataset.  Queryable fields include a cluster id for each vertex, as 
     well as the centers of the clusters.
 
-    An instance of this model can be created using
-    :func:`graphlab.kmeans.create`. Do NOT construct the model directly.
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.kmeans.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
     """
     def __init__(self, model):
         '''__init__(self)'''
@@ -233,7 +234,8 @@ get_default_options = _get_default_options_wrapper(
                           'kmeans', 
                           'KmeansModel')
 
-def create(dataset, num_clusters, features=None, max_iterations=10, verbose=True):
+def create(dataset, num_clusters=None, features=None, initial_centers=None,
+           max_iterations=10, verbose=True):
     r"""
     Run the k-means++ clustering algorithm, returning a KmeansModel object
     that contains the cluster centers and the cluster assignment for
@@ -244,12 +246,12 @@ def create(dataset, num_clusters, features=None, max_iterations=10, verbose=True
     cluster membership between iterations, the algorithm terminates. 
 
     Parameters
-    -----------
+    ----------
     dataset : SFrame
         Each row in the SFrame is an observation.
 
     num_clusters : int
-        Number of clusters.  This is the 'k' in k-means.
+        Number of clusters. This is the 'k' in k-means.
 
     features : list[string], optional
         Names of feature columns to use in computing distances between
@@ -268,9 +270,18 @@ def create(dataset, num_clusters, features=None, max_iterations=10, verbose=True
         Note that columns of type *list* are not supported. Convert them to
         array columns if all entries in the list are of numeric types.
 
+    initial_centers : SFrame, optional
+        If None (default), k-means++ intelligently chooses initial cluster
+        centers. Otherwise, the algorithm starts with the centers provided in
+        this SFrame. If this SFrame is provided, the ``num_clusters`` parameter
+        does not need to be specified. ``initial_centers`` must have the columns
+        specified in the ``features`` argument.
+
     max_iterations : int, optional
         The maximum number of iterations to run. Prints a warning if the
-        algorithm does not converge after max_iterations iterations.
+        algorithm does not converge after max_iterations iterations. If set to
+        0, the model returns clusters defined by the initial centers and
+        assignments to those centers.
 
     verbose : bool, optional
         If True, print model training progress to the screen.
@@ -289,6 +300,7 @@ def create(dataset, num_clusters, features=None, max_iterations=10, verbose=True
     ----------
     - `Wikipedia - k-means clustering
       <http://en.wikipedia.org/wiki/K-means_clustering>`_
+
     - Artuhur, D. and Vassilvitskii, S. (2007) `k-means++: The Advantages of
       Careful Seeding <http://ilpubs.stanford.edu:8090/778/1/2006-13.pdf>`_. In
       Proceedings of the Eighteenth Annual ACM-SIAM Symposium on Discrete
@@ -296,7 +308,6 @@ def create(dataset, num_clusters, features=None, max_iterations=10, verbose=True
 
     Examples
     --------
-
     >>> sf = graphlab.SFrame({
         "d1": [ 0.46973508, 0.0063261, 0.14143399, 0.35025834,
                 0.83728709, 0.81438336, 0.74205833, 0.36273747,
@@ -312,8 +323,8 @@ def create(dataset, num_clusters, features=None, max_iterations=10, verbose=True
                 0.37992791, 0.44700959]
         })
 
-    It's important to standardize our columns to get the best results
-    possible from the k-means algorithm.
+    It's important to standardize our columns to get the best results possible
+    from the k-means algorithm.
 
     >>> for col in ['d1', 'd2', 'd3', 'd4']:
             sf[col] = (sf[col] - sf[col].mean()) / sf[col].std()
@@ -323,25 +334,79 @@ def create(dataset, num_clusters, features=None, max_iterations=10, verbose=True
     _mt._get_metric_tracker().track('toolkit.kmeans.create')
 
     opts = {'model_name': 'kmeans',
-            'num_clusters': num_clusters,
             'max_iterations': max_iterations,
             'verbose': verbose}
 
+    ## Validate input dataset
     if not (isinstance(dataset, _SFrame)):
-        raise TypeError("Input 'dataset' must be an SFrame")
+        raise TypeError("Input 'dataset' must be an SFrame.")
 
     if dataset.num_rows() == 0 or dataset.num_cols() == 0:
         raise ValueError("Input 'dataset' has no data.")
 
-    features = _validate_feature_types(dataset, features, [_array, dict, int, float])
-    sf_features = dataset.select_columns(features)
-    opts["features"] = sf_features
+    ## Validate input initial centers
+    if initial_centers is not None:
+        if not (isinstance(initial_centers, _SFrame)):
+            raise TypeError("Input 'initial_centers' must be an SFrame.")
+
+        if initial_centers.num_rows() == 0 or initial_centers.num_cols() == 0:
+            raise ValueError("An 'initial_centers' argument is provided " +\
+                             "but has no data.")
+
+    ## Validate number of clusters
+    if initial_centers is None:
+        if num_clusters is None:
+            raise ValueError("Number of clusters cannot be determined from " +\
+                             "'num_clusters' or 'initial_centers'. You must " +\
+                             "specify one of these arguments.")
+        else:
+            _num_clusters = num_clusters
+    
+    else:
+        num_centers = initial_centers.num_rows()
+
+        if num_clusters is None:
+            _num_clusters = num_centers
+        else:
+            if num_clusters != num_centers:
+                raise ValueError("The value of 'num_clusters' does not match " +\
+                                 "the number of provided initial centers. " +\
+                                 "Please provide only one of these arguments " +\
+                                 "or ensure the values match.")
+            else:
+                _num_clusters = num_clusters
 
     num_examples = dataset.num_rows()
-    if num_clusters > num_examples:
-        _logging.warning("Clipping num_clusters to be the number of data points.\n")
-        num_clusters = num_examples
-        opts['num_clusters'] = num_examples
 
+    if _num_clusters > num_examples:
+        _logging.warning("'num_clusters' exceeds the number of data points. " +\
+                         "Resetting 'num_clusters' to be exactly the number of data points.")
+        _num_clusters = num_examples
+
+    opts['num_clusters'] = _num_clusters
+
+    ## Validate the features in the dataset
+    features = _select_valid_features(dataset, features, [_array, dict, int, float])
+    sf_features = dataset.select_columns(features)
+    opts['features'] = sf_features
+
+    ## Validate the features in the initial centers (if provided)
+    if initial_centers is not None:
+        try:
+            initial_centers = initial_centers.select_columns(features)
+        except:
+            raise ValueError("Specified features cannot be extracted from the " +\
+                             "provided initial centers.")
+
+        if initial_centers.column_types() != sf_features.column_types():
+            raise TypeError("Feature types are different in the dataset and " +\
+                             "initial centers.")
+
+    else:
+        initial_centers = _graphlab.SFrame()
+
+    opts['initial_centers'] = initial_centers
+
+    ## Create and return the model
     params = _graphlab.toolkits._main.run('kmeans_train', opts, verbose)
     return KmeansModel(params['model'])
