@@ -38,7 +38,6 @@ Create.
 """
 
 import graphlab.generic as _generic
-from . import _cxxnet_adaptor
 
 
 class _layer_types_enum():
@@ -71,11 +70,9 @@ class _LayerBase(_generic.CommonEqualityMixin):
     _name = None
     _type = None
 
-    def __init__(self, _name, _type, **kwargs):
+    def __init__(self, _name, _type):
         self._name = _name
         self._type = _type
-        for (k, v) in kwargs.iteritems():
-            setattr(self, k, v)
 
     def __repr__(self):
         ss = [self.__class__.__name__]
@@ -91,21 +88,21 @@ class _LayerBase(_generic.CommonEqualityMixin):
             ret += "  %s = %s\n" % (k, str(v))
         return ret
 
-    def _cxxnet_str(self):
-        ret = self._name + "\n"
-        key_values = dict([(k, self.__getattribute__(k)) for k in self.__dict__ if not k.startswith('_')])
-        for k, v in key_values.iteritems():
-            k = _cxxnet_adaptor.gl_to_cxxnet_map.get(str(k), str(k))
-            v = _cxxnet_adaptor.gl_to_cxxnet_map.get(str(v), str(v))
-            ret += "  %s = %s\n" % (k, v)
-        return ret
-
     def _verify(self):
         return True
 
     def _get_output_shape(self, input_shape):
         return input_shape
 
+    def _set_params(self, **kwargs):
+        '''
+        This is a back door to enable setting of arbitrary kwargs without
+        exposing the ability to pass bogus parameters to the user.
+        '''
+        for (k, v) in kwargs.iteritems():
+            setattr(self, k, v)
+        return
+
     @property
     def name(self):
         '''
@@ -192,9 +189,12 @@ class FullConnectionLayer(_LayerBase):
 
     >>> layer = graphlab.deeplearning.layers.FullConnectionLayer(20)
     """
-    def __init__(self, num_hidden_units, **kwargs):
-        super(FullConnectionLayer, self).__init__("fullc", types.CONNECTION_LAYER, **kwargs)
+    def __init__(self, num_hidden_units, init_bias=0, init_sigma=0.01, init_random='gaussian'):
+        super(FullConnectionLayer, self).__init__("fullc", types.CONNECTION_LAYER)
         self.num_hidden_units = num_hidden_units
+        self.init_bias = init_bias
+        self.init_sigma = init_sigma
+        self.init_random = init_random
 
     def _verify(self):
         if (self.num_hidden_units <= 0):
@@ -217,8 +217,8 @@ class FlattenLayer(_LayerBase):
     >>> layer = graphlab.deeplearning.layers.FlattenLayer()
 
     """
-    def __init__(self, **kwargs):
-        super(FlattenLayer, self).__init__("flatten", types.TRANSITION_LAYER, **kwargs)
+    def __init__(self):
+        super(FlattenLayer, self).__init__("flatten", types.TRANSITION_LAYER)
 
     def _get_output_shape(self, input_shape):
         n = input_shape[0] * input_shape[1] * input_shape[2]
@@ -231,7 +231,7 @@ class FlattenLayer(_LayerBase):
 # /*                                                                        */
 # /**************************************************************************/
 class _ActivationLayerBase(_LayerBase):
-    def __init__(self, **kwargs):
+    def __init__(self):
         super(_ActivationLayerBase, self).__init__(self._name, types.ACTIVATION_LAYER)
 
 
@@ -377,23 +377,18 @@ class ConvolutionLayer(_LayerBase):
     >>> layer = graphlab.deeplearning.layers.ConvolutionLayer(kernel_size=3, num_channels=10)
 
     """
-    def __init__(self, kernel_size, num_channels, **kwargs):
-        super(ConvolutionLayer, self).__init__("conv", types.CONVOLUTION_LAYER,
-                                               kernel_size=kernel_size,
-                                               num_channels=num_channels,
-                                               **kwargs)
+    def __init__(self, kernel_size, num_channels, stride=1, padding=0, num_groups=1, init_random='gaussian'):
+        super(ConvolutionLayer, self).__init__("conv", types.CONVOLUTION_LAYER)
+        self.kernel_size = kernel_size
+        self.num_channels = num_channels
+        self.stride = stride
+        self.padding = padding
+        self.num_groups = num_groups
+        self.init_random = init_random
 
     def _get_output_shape(self, input_shape):
-        if not hasattr(self, 'stride'):
-            stride = _LayerDefaults().STRIDE_DEFAULT
-        else:
-            stride = self.stride
-        if hasattr(self, 'padding'):
-            padding = self.padding
-        else:
-            padding = _LayerDefaults().PADDING_DEFAULT
-        return [(input_shape[0] + 2 * padding - self.kernel_size) / stride + 1,
-                (input_shape[1] + 2 * padding - self.kernel_size) / stride + 1,
+        return [(input_shape[0] + 2 * self.padding - self.kernel_size) / self.stride + 1,
+                (input_shape[1] + 2 * self.padding - self.kernel_size) / self.stride + 1,
                 self.num_channels]
 
     def _verify(self):
@@ -401,11 +396,11 @@ class ConvolutionLayer(_LayerBase):
             raise ValueError('kernel_size must be positive')
         if (self.num_channels <= 0):
             raise ValueError('num_channels must be positive')
-        if (hasattr(self, 'padding') and self.padding <= 0):
+        if (self.padding < 0):
             raise ValueError('padding must be positive')
-        if (hasattr(self, 'stride') and self.stride <= 0):
+        if (self.stride <= 0):
             raise ValueError('stride must be positive')
-        if (hasattr(self, 'num_groups') and self.num_groups <= 0):
+        if (self.num_groups <= 0):
             raise ValueError('num_groups must be positive')
         return super(ConvolutionLayer, self)._verify()
 
@@ -416,32 +411,25 @@ class ConvolutionLayer(_LayerBase):
 # /*                                                                        */
 # /**************************************************************************/
 class _PoolingLayerBase(_LayerBase):
-    def __init__(self, kernel_size, **kwargs):
+    def __init__(self, kernel_size, stride=1, padding=0):
         if (kernel_size <= 0):
             raise ValueError('kernel_size must be positive')
-        super(_PoolingLayerBase, self).__init__(self._name, types.POOLING_LAYER,
-                                                kernel_size=kernel_size, **kwargs)
+        super(_PoolingLayerBase, self).__init__(self._name, types.POOLING_LAYER)
+        self.kernel_size = kernel_size
+        self.stride = stride
+        self.padding = padding
+
 
     def _get_output_shape(self, input_shape):
-        if not hasattr(self, 'stride'):
-            stride = _LayerDefaults().STRIDE_DEFAULT
-        else:
-            stride = self.stride
-        if hasattr(self, 'padding'):
-            padding = self.padding
-        else:
-            padding = _LayerDefaults().PADDING_DEFAULT
-        return [(input_shape[0] + 2 * padding - self.kernel_size) / stride + 1, (input_shape[1] + 2 * padding - self.kernel_size) / stride + 1, input_shape[2]]
+        return [(input_shape[0] + 2 * self.padding - self.kernel_size) / self.stride + 1, (input_shape[1] + 2 * self.padding - self.kernel_size) / self.stride + 1, input_shape[2]]
 
     def _verify(self):
         if (self.kernel_size <= 0):
             raise ValueError('kernel_size must be positive')
-        if (hasattr(self, 'padding') and self.padding <= 0):
+        if (self.padding < 0):
             raise ValueError('padding must be positive')
-        if (hasattr(self, 'stride') and self.stride <= 0):
+        if (self.stride <= 0):
             raise ValueError('stride must be positive')
-        if (hasattr(self, 'num_groups') and self.num_groups <= 0):
-            raise ValueError('num_groups must be positive')
         return super(_PoolingLayerBase, self)._verify()
 
 
@@ -573,8 +561,8 @@ class DropoutLayer(_LayerBase):
     >>> layer = graphlab.deeplearning.layers.DropoutLayer()
 
     """
-    def __init__(self, threshold=0.5, **kwargs):
-        super(DropoutLayer, self).__init__("dropout", types.REGULARIZATION_LAYER, **kwargs)
+    def __init__(self, threshold=0.5):
+        super(DropoutLayer, self).__init__("dropout", types.REGULARIZATION_LAYER)
         self.threshold = threshold
 
     def _verify(self):
@@ -622,22 +610,25 @@ class LocalResponseNormalizationLayer(_LayerBase):
 
     """
 
-    def __init__(self, **kwargs):
-        super(LocalResponseNormalizationLayer, self).__init__("lrn", types.REGULARIZATION_LAYER, **kwargs)
+    def __init__(self, alpha=0.001, beta=0.75, local_size=5, knorm=1):
+        super(LocalResponseNormalizationLayer, self).__init__("lrn", types.REGULARIZATION_LAYER)
+        self.alpha = alpha
+        self.beta = beta
+        self.local_size = local_size
+        self.knorm = knorm
 
     def _verify(self):
-        if (hasattr(self, 'alpha') and self.alpha <= 0):
+        if (self.alpha <= 0):
             raise ValueError('alpha must be positive')
-        if (hasattr(self, 'beta') and self.beta <= 0):
+        if (self.beta <= 0):
             raise ValueError('beta must be positive')
-        if (hasattr(self, 'local_size') and (self.local_size <= 0 or self.local_size % 2 == 0)):
+        if (self.local_size <= 0 or self.local_size % 2 == 0):
             raise ValueError('local_size must be positive and odd')
-        if (hasattr(self, 'knorm') and self.knorm <= 0):
+        if (self.knorm <= 0):
             raise ValueError('knorm must be positive')
         return super(LocalResponseNormalizationLayer, self)._verify()
 
 
-
 # /**************************************************************************/
 # /*                                                                        */
 # /*                             Output Layers                              */
@@ -675,7 +666,7 @@ class SoftmaxLayer(_LayerBase):
     >>> layer = graphlab.deeplearning.layers.SoftmaxLayer()
 
     """
-    def __init__(self, **kwargs):
+    def __init__(self):
         super(SoftmaxLayer, self).__init__("softmax", types.OUTPUT_LAYER)
 
 
