@@ -42,9 +42,10 @@ class ConnectedComponentsModel(_ModelBase):
     | component_id   | An SFrame with each vertex's component id           |
     +----------------+-----------------------------------------------------+
 
-    An instance of this model can be created using
-    :func:`graphlab.connected_components.create`.  Do NOT construct the model
-    directly.
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.connected_components.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
 
     See Also
     --------
