@@ -40,9 +40,11 @@ class GraphColoringModel(_ModelBase):
     | num_colors     | Number of colors in the graph                       |
     +----------------+-----------------------------------------------------+
 
-    An instance of this model can be created using
-    :func:`graphlab.graph_coloring.create`.  Do NOT construct the model
-    directly.
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.graph_coloring.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
+
 
     See Also
     --------
