@@ -7,13 +7,57 @@ import graphlab as _graphlab
 from graphlab.toolkits._model import Model as _Model
 from graphlab.data_structures.sframe import SFrame as _SFrame
 import graphlab.toolkits._internal_utils as _tkutl
-import json as _json
-import logging as _logging
 from graphlab.toolkits._model import _get_default_options_wrapper as \
                                                   __get_default_options_wrapper
+import array
+import copy
+
+
+get_default_options = __get_default_options_wrapper('nearest_neighbors_ball_tree',
+                                                    'nearest_neighbors',
+                                                    'NearestNeighborsModel')
+
+
+def choose_auto_distance(feature_names, column_names, column_types):
+    """
+    Construct composite distance parameters based on selected features and their
+    types.
+    """
+
+    ## Make a dictionary from the column_names and column_types
+    col_type_dict = {k: v for k, v in zip(column_names, column_types)}
+
+    ## Loop through feature names, appending a distance component if the
+    #  feature's type is *not* numeric. If the type *is* numeric, append it to
+    #  the numeric_cols list, then at the end make a numeric columns distance
+    #  component.
+    composite_distance_params = []
+    numeric_cols = []
+
+    for c in feature_names:
+        if col_type_dict[c] == str:
+            composite_distance_params.append([[c], _graphlab.distances.levenshtein, 1])
+        elif col_type_dict[c] == dict:
+            composite_distance_params.append([[c], _graphlab.distances.jaccard, 1])
+        elif col_type_dict[c] == array.array:
+            composite_distance_params.append([[c], _graphlab.distances.euclidean, 1])
+        elif col_type_dict[c] == list:
+            composite_distance_params.append([[c], _graphlab.distances.euclidean, 1])
+        elif col_type_dict[c] in [int, float, array.array, list]:
+            numeric_cols.append(c)
+        else:
+            raise TypeError("Unable to automatically determine a distance "+\
+                "for column {}".format(c))
+
+    # Make the standalone numeric column distance component
+    if len(numeric_cols) > 0:
+        composite_distance_params.append([numeric_cols, _graphlab.distances.euclidean, 1])
+
+    return composite_distance_params
+
 
 def create(dataset, label=None, features=None, distance='auto', method='auto',
-    verbose=True, **kwargs):
+    composite_params=None, verbose=True, **kwargs):
     """
     Create a nearest neighbor model, which can be searched efficiently and
     quickly for the nearest neighbors of a query observation. If the `method`
@@ -47,18 +91,28 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
         - *Dictionary*: key-value pairs with numeric (integer or float) values.
           Each key indicates a separate variable in the model.
 
-        Columns of type *list* are not supported. Convert them to array columns
-        if all entries in the list are of numeric types.
+        - *String*: string values.
 
-    distance : string, optional
+        Columns of type *list* are not supported. Convert them to array columns
+        if all entries in the list are of numeric types. Please note: if
+        `composite_params` is also specified, this parameter is ignored.
+
+    distance : string or function, optional
         Name of the function that measures the distances between two
         observations. Please see the notes and references for detailed
-        descriptions of the distances. Note that for sparse vectors, euclidean,
-        manhattan, cosine, and weighted_jaccard distance assume missing keys
-        have value 0.0.
+        descriptions of the distances. Note that for sparse vectors, missing
+        keys are assumed to have value 0.0. Please note: if `composite_params`
+        is also specified, this parameter is ignored.
 
         - *auto* (default): the model chooses a reasonable distance based on the
-          data types in 'dataset'.
+          data types in 'dataset'. Columns of type str will be compared using
+          levenshtein distance, columns of type dict use jaccard distance, and
+          columns of type float, int, or list will be combined and use
+          euclidean distance. The set of column-specific distances are
+          aggregated into a single composite distance.
+
+        - *squared_euclidean*: works only with the `brute_force` method because
+          it is not a metric.
 
         - *euclidean*
 
@@ -78,6 +132,11 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
           true metric. Please see `Wikipedia
           <http://en.wikipedia.org/wiki/Cosine_similarity>`_ for more detail.
 
+        - *dot_product*: works only with the 'brute_force' method because it is
+          not a true metric.
+
+        - *levenshtein*: for a single column of string inputs.
+
     method : {'auto', 'ball_tree', 'brute_force'}, optional
         Method for computing nearest neighbors. The options are:
 
@@ -90,8 +149,8 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
         - *ball_tree*: use a tree structure to find the k-closest neighbors to
           each query point. The ball tree model is slower to construct than the
           brute force model, but queries are faster than linear time. This
-          method is not yet implemented for the jaccard distances and is not
-          applicable for cosine distance. See `Liu, et al (2004)
+          method is not applicable for the cosine and dot product distances. See
+          `Liu, et al (2004)
           <http://papers.nips.cc/paper/2666-an-investigation-of-p
           ractical-approximat e-nearest-neighbor-algorithms>`_ for
           implementation details.
@@ -101,6 +160,16 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
           creation with the brute force method (although the reference data is
           held in the model, but each query takes linear time.
 
+    composite_params : list [list [list [string], string or function, float]]
+        Multiple sets of features and corresponding distance functions can be
+        used as inputs to a composite distance function. Each element of this
+        composite is specified by a list in this argument. Each inner list must
+        include a list of feature names, the name of a distance function, and a
+        relative weight. See the examples and notes sections below. If
+        `composite_params` is specified, any standalone `features`, `distance`,
+        and `method` arguments are ignored. Keyword arguments are applied to
+        each member of the composite distance computation.
+
     verbose: bool, optional
         If True, print progress updates and model details.
 
@@ -132,12 +201,10 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
       divide by the standard deviation.
 
     - Distance definitions. Suppose :math:`u` and :math:`v` are observations
-      with :math:`d` variables each.  For the jaccard distances, suppose
-      :math:`S` and :math:`T` are the sets of keys from two observations'
-      dictionaries. For the weighted version of jaccard distance, suppose
-      :math:`S_k` and :math:`T_k` are the values associated with key :math:`k`
-      in the respective dictionaries. Typically these values are counts, i.e.
-      of words or n-grams.
+      with :math:`d` variables each.
+
+        - `squared_euclidean`
+            .. math:: D(u, v) = \sum_i^d (u_i - v_i)^2
 
         - `euclidean`
             .. math:: D(u, v) = \\sqrt{\sum_i^d (u_i - v_i)^2}
@@ -145,6 +212,23 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
         - `manhattan`
             .. math:: D(u, v) = \\sum_i^d |u_i - v_i|
 
+        - `cosine`
+            .. math::
+
+                D(u, v) = 1 - \\frac{\sum_i^d u_i v_i}
+                {\sqrt{\sum_i^d u_i^2}\sqrt{\sum_i^d v_i^2}}
+
+        - `dot_product`
+            .. math::
+
+                D(u, v) = \\frac{1}{\sum_i^d u_i v_i}
+
+    - For the jaccard distances, suppose :math:`S` and :math:`T` are the sets of
+      keys from two observations' dictionaries. For the weighted version of
+      jaccard distance, suppose :math:`S_k` and :math:`T_k` are the values
+      associated with key :math:`k` in the respective dictionaries. Typically
+      these values are counts, i.e. of words or n-grams.
+
         - `jaccard`
             .. math:: D(S, T) = 1 - \\frac{|S \cap T|}{|S \cup T|}
 
@@ -154,11 +238,35 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
                 D(S, T) = 1 - \\frac{\sum_{k \in S \cup T} \min\{S_k, T_k\}}
                 {\sum_{k \in S \cup T} \max\{S_k, T_k\}}
 
-        - `cosine`
-            .. math::
+    - Levenshtein distance is a type of edit distance for string types. The
+      distance is the number of insertion, deletion, and substituion edits
+      needed to transform string :math:`A` into string :math:`B`.
 
-                D(u, v) = 1 - \\frac{\sum_i^d u_i v_i}
-                {\sqrt{\sum_i^d u_i^2}\sqrt{\sum_i^d v_i^2}}
+        .. math::
+
+            D(A, B) = d(|A|, |B|)
+
+        .. math ::
+
+            d(i, j) = \max(i, j), \quad \mathrm{if } \min(i, j) = 0
+
+        .. math ::
+
+            d(i, j) = \min \Big \{d(i-1, j) + 1, \ d(i, j-1) + 1, \ d(i-1, j-1) + I(A_i \\neq B_i) \Big \}, \quad \mathrm{else}
+
+    - Composite distances are simply weighted sums of the above distances. The
+      set of features input to each component distance may vary, and the weight
+      on each component acts as a multiplier before each of the component
+      distances is summed. For example, if ``composite_params`` is set to
+
+      ``[[['X1', 'X2'], 'euclidean', 2], [['X2', 'X3'], 'manhattan', 3]]``,
+
+      then the overall distance computation for rows :math:`a` and :math:`b` is:
+
+        .. math::
+
+            D(a, b) = 2 * D_{euclidean}(a[[X1, X2]], b[[X1, X2]]) +
+            3 * D_{manhattan}(a[[X2, X3]], b[[X2, X3]])
 
     References
     ----------
@@ -185,20 +293,32 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
     - `Wikipedia - Cosine distance
       <http://en.wikipedia.org/wiki/Cosine_similarity>`_
 
+    - `Wikipedia - Levenshtein distance
+      <http://en.wikipedia.org/wiki/Levenshtein_distance>`_
+
     Examples
     --------
     Construct a nearest neighbors model with automatically determined method and
     distance:
 
-    >>> sf = graphlab.SFrame({'feature1': [0.98, 0.62, 0.11],
-    ...                       'feature2': [0.69, 0.58, 0.36]})
-    >>> model = graphlab.nearest_neighbors.create(sf, 'label')
+    >>> sf = graphlab.SFrame({'X1': [0.98, 0.62, 0.11],
+    ...                       'X2': [0.69, 0.58, 0.36],
+                              'str_feature': ['cat', 'dog', 'fossa']})
+    >>> model = graphlab.nearest_neighbors.create(sf, features=['X1', 'X2'])
 
     For datasets with a large number of rows and up to about 100 variables, the
     ball tree method often leads to much faster queries.
 
-    >>> model = graphlab.nearest_neighbors.create(sf, distance='manhattan',
-    ...                                           method='ball-tree')
+    >>> model = graphlab.nearest_neighbors.create(sf, features=['X1', 'X2'],
+    ...                                           method='ball_tree')
+
+    Often the final determination of a neighbor is based on several distance
+    computations over different sets of features. Each part of this composite
+    distance may have a different relative weight.
+
+    >>> model = graphlab.nearest_neighbors.create(sf, composite_params=[
+    ...                                          [['X1', 'X2'], 'euclidean', 2.],
+    ...                                          [['str_feature'], 'levenshtein', 3.]])
     """
 
     ## Validate the 'dataset' input
@@ -206,88 +326,193 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
     _tkutl._raise_error_if_sframe_empty(dataset, "dataset")
 
 
-    ## Validate and preprocess the 'features' input
-    if features is None:
-        _features = dataset.column_names()
-
-    else:
-        _features = features[:]  # copy to prevent modifying original input
-        _tkutl._raise_error_if_not_iterable(_features, "features")
-
-        if not all([isinstance(x, str) for x in _features]):
-            raise TypeError("Input 'features' must contain only strings.")
-
-    sf_features = _tkutl._toolkits_select_columns(dataset, _features)
+    ## Exclude inappropriate combinations of method an distance
+    if method == 'ball_tree' and (distance == 'cosine'
+                                  or distance == _graphlab.distances.cosine
+                                  or distance == 'dot_product'
+                                  or distance == _graphlab.distances.dot_product):
+        raise TypeError("The ball tree method does not work with 'cosine' " +\
+                        "or 'dot_product' distance. Please use the 'brute_force' " +\
+                        "method for these distances.")
 
 
-    ## Validate and preprocess the 'label' input
+    ## Initial validation and processing of the label
     if label is None:
-        ref_label = '__id'
+        _label = '__id'
 
-        ## remove any feature that has the same name as the label
         try:
-            sf_features.remove_column(ref_label)
+            _dataset = dataset.add_row_number(column_name=_label)
         except:
-            pass
-        else:
-            if features is not None:
-                _logging.warning("Feature column '{}' was removed because ".format(ref_label) +\
-                                 "because it conflicts with the automatically " +\
-                                 "supplied row number column.")
-
-        ## create a label SFrame with the row numbers            
-        sf_features = sf_features.add_row_number(column_name=ref_label)
-        sf_label = sf_features[[ref_label]]
-        sf_features.remove_column(ref_label)
+            print "Tried to add a default label column '{}' ".format(_label) +\
+                  "but a column by this name already exists. Using the " + \
+                  "existing column as the label column."
+            _dataset = dataset
 
     else:
-
-        ## validate the label argument
         if not label in dataset.column_names():
-            raise ValueError("Input 'label' must be the name of a column in the " +\
-                             "reference SFrame 'dataset'.")
+            raise ValueError(
+                "Input 'label' must be a string matching the name of a " +\
+                "column in the reference SFrame 'dataset'.")
 
         if not dataset[label].dtype() == str and not dataset[label].dtype() == int:
             raise TypeError("The label column must contain integers or strings.")
 
-        ## remove any feature that has the same name as the label
-        try:
-            sf_features.remove_column(label)
-        except:
-            pass
-        else:
-            if features is not None:
-                _logging.warning("Feature column '{}' was removed because ".format(label) +\
-                                 "it conflicts with the label column.")
+        _label = label
+        _dataset = dataset
 
-        ## create the SFrame with row labels
-        sf_label = _tkutl._toolkits_select_columns(dataset, [label])
+    sf_label = _tkutl._toolkits_select_columns(_dataset, [_label])
 
 
-    ## Get feature types and decide which method to use
-    num_variables = sum([len(x) if hasattr(x, '__iter__') else 1
-                        for x in sf_features[0].itervalues()])  # assume the number of list elements does not change
-
-    if method == 'auto':
-        if ((distance=='euclidean' or distance=='manhattan' or distance=='auto')
-                and not dict in sf_features.column_types()
-                and num_variables <= 100):
-            _method = 'ball_tree'
-        else:
-            _method = 'brute_force'
+    ## Clean the method options and create the options dictionary
+    if len(kwargs) > 0:
+        _method_options = {k.lower(): v for k, v in kwargs.items()}
     else:
-        _method = method
+        _method_options = {}
 
 
-    if _method == 'ball_tree' and distance == 'cosine':
-        raise TypeError("The ball tree method does not work with cosine " +\
-                        "distance. Please try 'euclidean' or 'manhattan' " +\
-                        "distance instead.")
 
-    if _method == 'ball_tree' and distance == 'jaccard':
-        raise TypeError("The ball tree method does not currently work with " +\
-                        "jaccard distance. Please try 'euclidean' or 'manhattan' " +\
-                        "distance instead.")
+    ## If composite inputs aren't specifed, formulate the standalone inputs as a
+    #  composite input for code simplicity. If the standalone input doesn't
+    #  specify features, choose all the features and make a set of distance
+    #  components based on feature type.
+    if composite_params is None:
+
+        ## If not features specified, use them all
+        if features is None:
+            _features = [x for x in _dataset.column_names() if x != _label]
+        else:
+            _features = features[:]
+
+        ## If the distance argument is 'auto', turn it into a list of distance
+        #  components by choosing an automatic distance for each feature based
+        #  on its type.
+        if distance == 'auto':
+            _composites = choose_auto_distance(_features,
+                                               dataset.column_names(),
+                                               dataset.column_types())
+
+        else:
+            _composites = [[_features, distance, 1]]
+
+
+    # Ignore automatically generated components if components have been provided
+    else:
+        if distance != 'auto':
+            raise ValueError(
+                "Either the 'distance' parameter or the 'composite_params' " +\
+                "parameter may be specified, but not both.")
+
+        if features is not None:
+            raise ValueError(
+                "Either the 'features' parameter or the 'composite_params' " +\
+                "parameter may be specified, but not both.")
+
+        if len(composite_params) == 0:
+            raise ValueError(
+                "'composite_params' was specified as an empty list. If " +\
+                "specified, this parameter must contain at least one distance " +\
+                "component, which is a list containing three elements: a list " +\
+                "of feature names, a distance name or function, and a relative " +\
+                "weight.")
+
+        _composites = copy.deepcopy(composite_params)
+
+
+
+    ## Clean the list of features in each component of the composite inputs, and
+    #  compile the union of the lists of features.
+    all_features = []
+
+    for i in range(len(_composites)):
+
+        if len(_composites[i]) != 3:
+            raise ValueError("Each element of 'composite_params' must be a " +\
+                             "list with three members.")
+
+        feature_names = _composites[i][0]
+        if len(feature_names) == 0:
+            raise ValueError("An empty list of features cannot be passed " +\
+                             "as part of a composite distance function.")
+
+        # set of features must be iterable
+        _tkutl._raise_error_if_not_iterable(feature_names, "features")
+
+        # feature names must be strings
+        if not all([isinstance(x, str) for x in feature_names]):
+            raise TypeError("Input 'features' must contain only strings.")
+
+        # remove the label name from all of the features lists
+        feature_names = [x for x in feature_names if x != _label]
+
+        # ensure that string features are in single columns
+        if len(feature_names) > 1 and any([_dataset[x].dtype() is str for x in feature_names]):
+            raise ValueError(                             
+                "Multiple features have been entered, one of which is of string " +\
+                "type. If the input features for any distance component contain a " +\
+                "string column, that must be the only column for that component.")
+
+        # ensure that relative weights are integers or floats
+        if not isinstance(_composites[i][2], (int, float)):
+            raise ValueError(
+                "The weight of each distance component must be a single " +\
+                "integer or a float value.")
+
+        # combine all features into a big list
+        _composites[i][0] = feature_names
+        all_features += feature_names
+
+        # convert distance strings to distance functions
+        temp_dist = _composites[i][1]
+        if isinstance(temp_dist, str):
+            _composites[i][1] = _graphlab.util._get_distance(temp_dist)
+
+
+    # Pull out the relevant features from the input dataset (the union of
+    # features over all distance components)
+    all_features = list(set(all_features))
+    sf_features = _tkutl._toolkits_select_columns(_dataset, all_features)
+
+
+    ## Decide which method to use
+    ## - If more than one distance component (specified either directly or
+    #  generated automatically because distance set to 'auto'), then do brute
+    #  force.
+    if len(_composites) > 1:
+        _method = 'brute_force'
+
+        if method == 'ball_tree':
+            print "Defaulting to brute force instead of ball tree because " +\
+                "there are multiple distance components."
+
+    else:
+
+        if method == 'auto':
+
+            # get the total number of variables. Assume the number of elements in
+            # array type columns does not change
+            num_variables = sum([len(x) if hasattr(x, '__iter__') else 1
+                for x in sf_features[0].itervalues()])
+
+            # flag if all the features in the single composite are of numeric
+            # type.
+            numeric_type_flag = all([x in [int, float, list, array.array]
+                for x in sf_features.column_types()])
+
+            ## Conditions necessary for ball tree to work and be worth it
+            if ((_composites[0][1] in ['euclidean',
+                                       'manhattan',
+                                       _graphlab.distances.euclidean,
+                                       _graphlab.distances.manhattan])
+                    and numeric_type_flag is True
+                    and num_variables <= 100):
+
+                    _method = 'ball_tree'
+
+            else:
+                _method = 'brute_force'
+
+        else:
+            _method = method
 
 
     ## Pick the right model name for the method
@@ -303,44 +528,40 @@ def create(dataset, label=None, features=None, distance='auto', method='auto',
         raise ValueError("Method must be 'brute_force', 'ball_tree', or 'auto'")
 
 
-    ## Clean the method options and create the options dictionary
-    if len(kwargs) > 0:
-        _method_options = {k.lower(): v for k, v in kwargs.items()}
-    else:
-        _method_options = {}
-
+    ## Package the model options
     opts = {}
     opts.update(_method_options)
     opts.update(
         {'model_name': model_name,
-        'label': sf_label,
-        'features': sf_features,
-        'distance': distance})
+        'sf_label': sf_label,
+        'sf_features': sf_features,
+        'composite_params': _composites})
 
     ## Construct the nearest neighbors model
-    print "Starting model construction..."
-    result = _graphlab.toolkits._main.run('nearest_neighbors_train', opts,
-                                          verbose)
+    if verbose:
+        print "Starting model construction..."
+
+    result = _graphlab.extensions._nearest_neighbors.train(opts)
     model_proxy = result['model']
     model = NearestNeighborsModel(model_proxy)
 
-    if verbose is True:
+    if verbose:
         model.summary()
         print
 
     return model
 
-get_default_options = __get_default_options_wrapper('nearest_neighbors_ball_tree', 
-                                                    'nearest_neighbors', 
-                                                    'NearestNeighborsModel')
-
 
 class NearestNeighborsModel(_Model):
     """
     The NearestNeighborsModel represents rows of an SFrame in a structure that
     is used to quickly and efficiently find the nearest neighbors of a query
-    point. Use the 'create' method in this module to construct a
-    NearestNeighborsModel instance.
+    point.
+
+    This model cannot be constructed directly.  Instead, use
+    :func:`graphlab.nearest_neighbors.create` to create an instance
+    of this model. A detailed list of parameter options and code samples
+    are available in the documentation for the create function.
     """
 
     def __init__(self, model_proxy):
@@ -455,7 +676,7 @@ class NearestNeighborsModel(_Model):
         _mt._get_metric_tracker().track('toolkit.nearest_neighbors.list_fields')
 
         opts = {'model': self.__proxy__, 'model_name': self.__name__}
-        response = _graphlab.toolkits._main.run('nearest_neighbors_list_keys',
+        response = _graphlab.toolkits._main.run('_nearest_neighbors.list_keys',
                                                opts)
 
         return sorted(response.keys())
@@ -526,11 +747,11 @@ class NearestNeighborsModel(_Model):
         opts = {'model': self.__proxy__,
                 'model_name': self.__name__,
                 'field': field}
-        response = _graphlab.toolkits._main.run('nearest_neighbors_get_value',
+        response = _graphlab.toolkits._main.run('_nearest_neighbors.get_value',
                                                 opts)
-
         return response['value']
 
+
     def get_current_options(self):
         """
         Return a dictionary with the options used to define and create the
@@ -562,7 +783,7 @@ class NearestNeighborsModel(_Model):
         opts = {'model': self.__proxy__, 'model_name': self.__name__}
 
         return _graphlab.toolkits._main.run(
-            'nearest_neighbors_get_current_options', opts)
+            '_nearest_neighbors.get_current_options', opts)
 
     def _training_stats(self):
         """
@@ -601,7 +822,7 @@ class NearestNeighborsModel(_Model):
             'toolkit.nearest_neighbors.training_stats')
 
         opts = {'model': self.__proxy__, 'model_name': self.__name__}
-        return _graphlab.toolkits._main.run("nearest_neighbors_training_stats",
+        return _graphlab.toolkits._main.run("_nearest_neighbors.training_stats",
                 opts)
 
     def query(self, dataset, label=None, k=5, radius=None, verbose=True):
@@ -613,7 +834,7 @@ class NearestNeighborsModel(_Model):
 
         Parameters
         ----------
-        dataset : SFrame            
+        dataset : SFrame
             Query data. Must contain columns with the same names and types as
             the features used to train the model. Additional columns are
             allowed, but ignored. Please see the nearest neighbors
@@ -689,12 +910,10 @@ class NearestNeighborsModel(_Model):
             'toolkit.nearest_neighbors.query')
 
         ## Validate the 'dataset' input
-        _tkutl._raise_error_if_not_sframe(dataset,
-                                                                      "dataset")
-        _tkutl._raise_error_if_sframe_empty(dataset,
-                                                                        "dataset")
+        _tkutl._raise_error_if_not_sframe(dataset, "dataset")
+        _tkutl._raise_error_if_sframe_empty(dataset, "dataset")
 
-        ## Get model features        
+        ## Get model features
         ref_features = self.get('features')
         sf_features = _tkutl._toolkits_select_columns(dataset, ref_features)
 
@@ -705,11 +924,12 @@ class NearestNeighborsModel(_Model):
             sf_features = sf_features.add_row_number(column_name=ref_label)
             sf_label = sf_features[[ref_label]]
             sf_features.remove_column(ref_label)
-        
+
         else:
             if not label in dataset.column_names():
-                raise ValueError("Input 'label' must be the name of a column in the " +\
-                                 "reference SFrame 'dataset'.")
+                raise ValueError(
+                    "Input 'label' must be a string matching the name of a " +\
+                    "column in the reference SFrame 'dataset'.")
 
             if not dataset[label].dtype() == str and not dataset[label].dtype() == int:
                 raise TypeError("The label column must contain integers or strings.")
@@ -753,8 +973,10 @@ class NearestNeighborsModel(_Model):
                 'k': k,
                 'radius': radius}
 
-        print "Starting model querying..."
-        result = _graphlab.toolkits._main.run('nearest_neighbors_query', opts,
+        if verbose is True:
+            print "Starting model querying..."
+
+        result = _graphlab.toolkits._main.run('_nearest_neighbors.query', opts,
                                              verbose)
         return _SFrame(None, _proxy=result['neighbors'])
 
