@@ -3,13 +3,13 @@ The GraphLab Create recommender toolkit provides a unified interface to train a
 variety of recommender models and use them to make recommendations.
 
 Recommender models can be created using :meth:`graphlab.recommender.create` or
-loaded from a previously saved model using :meth:`graphlab.load_model`.
-The input data must be an SFrame with a column containing to user ids, a
-column containing to item ids, and optionally a column containing target values
-such as movie ratings, etc. When a target is not provided (as is the case in
-implicit feedback settings), then a collaborative filtering model based on
-item-item similarity is returned. For more details, please see the
-documentation for :meth:`graphlab.recommender.create`.
+loaded from a previously saved model using :meth:`graphlab.load_model`. The
+input data must be an SFrame with a column containing user ids, a column
+containing item ids, and optionally a column containing target values such as
+movie ratings, etc. When a target is not provided (as is the case in implicit
+feedback settings), then a collaborative filtering model based on item-item
+similarity is returned. For more details, please see the documentation for
+:meth:`graphlab.recommender.create`.
 
 A recommender model object can perform key tasks including
 `predict`, `recommend`, `evaluate`, and `save`. Model attributes and
