@@ -4,6 +4,8 @@ import random as _random
 import array as _array
 import logging as _logging
 from graphlab.toolkits._model import Model as _Model
+from graphlab.toolkits._internal_utils import _toolkit_repr_print
+from graphlab.toolkits._internal_utils import _precomputed_field
 
 import graphlab.aggregate as _Aggregate
 from graphlab.data_structures.sarray import SArray as _SArray
@@ -24,8 +26,9 @@ def _create(observation_data,
            ranking=True,
            verbose=True):
     r"""
-    A unified interface for training recommender models. The trained model can
-    be used to predict ratings and make recommendations.
+    A unified interface for training recommender models. Based on simple 
+    characteristics of the data, a type of model is selected and trained. The 
+    trained model can be used to predict ratings and make recommendations.
 
     To use specific options of a desired model, use the ``create`` function
     of the corresponding model.
@@ -53,19 +56,18 @@ def _create(observation_data,
         item id.
 
     target : string, optional
-        The `observation_data` can optionally contain a column of scores
-        representing ratings given by the users. If present, the name of this
-        column may be specified variables `target`.
+        Name of the column in `observation_data` containing ratings given by
+        users to items, if applicable.
 
     user_data : SFrame, optional
-        Side information for the users.  This SFrame must have a column with
-        the same name as what is specified by the `user_id` input parameter.
+        Side information for the users. This SFrame must have a column with the
+        same name as what is specified by the `user_id` input parameter.
         `user_data` can provide any amount of additional user-specific
         information.
 
     item_data : SFrame, optional
-        Side information for the items.  This SFrame must have a column with
-        the same name as what is specified by the `item_id` input parameter.
+        Side information for the items. This SFrame must have a column with the
+        same name as what is specified by the `item_id` input parameter.
         `item_data` can provide any amount of additional item-specific
         information.
 
@@ -77,12 +79,11 @@ def _create(observation_data,
 
     Returns
     -------
-    out :  A trained model, one of
-        :class:`graphlab.recommender.item_similarity_recommender.ItemSimilarityRecommender`,
-        :class:`graphlab.recommender.factorization_recommender.FactorizationRecommender`,
-        :class:`graphlab.recommender.ranking_factorization_recommender.RankingFactorizationRecommender`,
-        or
-        :class:`graphlab.recommender.popularity_recommender.PopularityRecommender`.
+    out :  A trained model.
+        - If a target column is given, then
+          :class:`graphlab.recommender.factorization_recommender.FactorizationRecommender`.
+        - If no target column is given, then 
+          :class:`graphlab.recommender.item_similarity_recommender.ItemSimilarityRecommender`.
 
     Examples
     --------
@@ -114,7 +115,7 @@ def _create(observation_data,
     for individual models. Such options can be passed to the underlying model
     just like any other parameter. For example, the following code creates
     an :class:`~graphlab.recommender.ItemSimilarityRecommender` with a space-saving
-    option called `only_top_k`. The returned model stores only the 25 most
+    option called `only_top_k`. The returned model stores only the 2 most
     similar items for item:
 
     >>> from graphlab.recommender import item_similarity_recommender
@@ -854,78 +855,50 @@ class _Recommender(_Model):
         stats = self.list_fields()
         options = self.get_current_options()
 
-        # Print model type and data schema
-        ret = self.__class__.__name__ + '\n'
+        section_names = []
+        sections = []
 
-        # If the model is untrained, there will not be an elapsed_time
-        is_trained = 'training_time' in stats
+        observation_columns = set(self.get('observation_data_column_names'))
+        not_needed = set([self.get('user_id'),
+                          self.get('item_id'),
+                          self.get('target')])
+        num_obs_fields = len(observation_columns.difference(not_needed))
 
-        # TODO: Refactor into functions
-        if is_trained:
+        num_user_features = 0
+        if 'user_side_data_column_names' in stats:
+            num_user_features = len(self.get('user_side_data_column_names'))
+        num_item_features = 0
+        if 'item_side_data_column_names' in stats:
+            num_item_features = len(self.get('item_side_data_column_names'))
 
-            ret += "\nSchema\n"
-            for k in ["user_id", "item_id", "target"]:
-                if k in options.keys():
-                    if options[k] not in ['', 'not_specified'] and options[k] is not None:
-                        ret += "  %-14s %s\n" % \
-                            (k.replace('_', ' ') + ':', options[k])
-                    del options[k]
+        section_names.append("Schema")
+        schema_fields = [
+            ('User ID', 'user_id'),
+            ('Item ID', 'item_id'),
+            ('Target', 'target'),
+            ('Additional observation features', _precomputed_field(num_obs_fields)),
+            ('Number of user side features', _precomputed_field(num_user_features)),
+            ('Number of item side features', _precomputed_field(num_item_features))]
+        sections.append(schema_fields)
 
-            # Statistics about the training and validation set
-            ret += "\nStatistics\n"
-            ks = ["num_observations",
-                  "num_users",
-                  "num_items"]
-            if all([k in stats for k in ks]):
-                ret += "  %-17s %11d obs %11d users %11d items\n" % \
-                    ("Training set:", self.get(ks[0]), self.get(ks[1]), self.get(ks[2]))
+        data_fields = [
+            ('Number of observations', 'num_observations'),
+            ('Number of users', 'num_users'),
+            ('Number of items', 'num_items')]
+        section_names.append("Statistics")
+        sections.append(data_fields)
 
-            if 'user_side_data_column_names' in stats:
-                if len(self.get('user_side_data_column_names')) > 0:
-                    ret += "  user side data:\n"
-                colnames = self.get('user_side_data_column_names')
-                coltypes = self.get('user_side_data_column_types')
-                assert len(colnames) == len(coltypes), \
-                    "Error importing user side data: mismatch between the \
-                     number of column names and column types."
+        training_fields = [
+            ('Training time', 'training_time')]
 
-                for i in range(len(colnames)):
-                    ret += '    {0} : {1}\n'.format(colnames[i], coltypes[i])
-
-            if 'item_side_data_column_names' in stats:
-                if len(self.get('item_side_data_column_names')) > 0:
-                    ret += "  item side data:\n"
-                colnames = self.get('item_side_data_column_names')
-                coltypes = self.get('item_side_data_column_types')
-                assert len(colnames) == len(coltypes), \
-                    "Error importing item side data: mismatch between the \
-                     number of column names and column types."
-
-                for i in range(len(colnames)):
-                    ret += '    {0} : {1}\n'.format(colnames[i], coltypes[i])
-
-
-            # Training summary
-            ks = ["training_time",
-                  "random_seed"]
-            if any([k in stats]):
-                ret += "\nTraining summary\n"
-            if 'elapsed_time' in stats:
-                ret += "  %-20s %fs\n" % ("training time:", self.get("elapsed_time"))
-            if 'data_load_elapsed_time' in stats:
-                ret += "  %-20s %fs\n" % ("data load time:", self.get("data_load_elapsed_time"))
-
-            if "validation_metrics_elapsed_time" in stats:
-                ret += "  %-20s %fs\n" % \
-                           ("metrics time:", self.get("validation_metrics_elapsed_time"))
-
-            # If available, print performance statistics
-            for k in stats:
-                if 'rmse' in k or 'precision' in k or 'recall' in k:
-                    ret += '  %-20s %-10s\n' % (k + ':', self.get(k))
-
-        else:
-            ret += '\nThis model has yet to be trained.\n'
+        if 'data_load_elapsed_time' in stats:
+            training_fields.append(('Data load time',
+                                    'data_load_elapsed_time'))
+        if 'validation_metrics_elapsed_time' in stats:
+            training_fields.append(('Validation metrics time',
+                                   'validation_metrics_elapsed_time'))
+        section_names.append("Training summary")
+        sections.append(training_fields)
 
         # Remove any options that should not be shown under "Settings"
         to_ignore = ['random_seed',
@@ -935,15 +908,15 @@ class _Recommender(_Model):
         for k in to_ignore:
             if k in options:
                 del options[k]
+        option_fields = []
+        for k, v in options.iteritems():
+            option_fields.append((k, _precomputed_field(v)))
 
-        # Print remaining hyperparameters
-        # TODO: Get max width of keys and use it for fixed width formatting.
-        if len(options) > 0:
-            ret += "\nSettings\n"
-            for k, v in options.iteritems():
-                ret += "  %-22s %-30s\n" % (k.replace('_', ' ') + ':', str(v))
+        section_names.append("Settings")
+        sections.append(option_fields)
 
-        return ret
+        width = 32
+        return _toolkit_repr_print(self, sections, width, section_names)
 
     def get_current_options(self):
         """
@@ -1120,27 +1093,40 @@ class _Recommender(_Model):
                   exclude_known=True,
                   verbose=True):
         """
-        Recommends the k-highest scored items for each user in users.
+        Recommend the ``k`` highest scored items for each user.
 
         Parameters
         ----------
-        users :  SArray or list; optional
-            An :class:`~graphlab.SArray` (or list) of users for which
-            to make recommendations.  If 'None', then recommend(...)
-            generates predictions for all users in the training set.
-            tHE type of the SArray or list must be the same as the type
-            of the user_id column in the training data.
+        users :  SArray, SFrame, or list, optional
+            Users or observation queries for which to make recommendations.
+            For list, SArray, and single-column inputs, this is simply a set
+            of user IDs. By default, recommendations are returned for all
+            users present when the model was trained. However, if the
+            recommender model was created with additional features in the
+            ``observation_data`` SFrame, then a corresponding SFrame of
+            observation queries -- observation data without item or target
+            columns -- can be passed to this method. For example, a model
+            trained with user ID, item ID, time, and rating columns may be
+            queried using an SFrame with user ID and time columns.  In this
+            case, the user ID column must be present, and all column names
+            should match those in the ``observation_data`` SFrame passed to
+            ``create.``
 
         k : int, optional
             The number of recommendations to generate for each user.
 
-        items : SArray, optional
-            To restrict the set of items under consideration, the user
-            may pass in an SArray of items from which the recommended
-            items will be chosen.  This allows one to choose only
-            items that are, for instance, in a particular genre or
-            category.  The default is that all items are under
-            consideration.
+        items : SArray, SFrame, or list, optional
+            Restricts the items from which recommendations can be made. If
+            ``items`` is an SArray, list, or SFrame with a single column,
+            only items from the given set will be recommended. This can be
+            used, for example, to restrict the recommendations to items
+            within a particular category or genre. If ``items`` is an
+            SFrame with user ID and item ID columns, then the item
+            restriction is specialized to each user. For example, if
+            ``items`` contains 3 rows with user U1 -- (U1, I1), (U1, I2),
+            and (U1, I3) -- then the recommendations for user U1 are
+            chosen from items I1, I2, and I3. By default, recommendations
+            are made from all items present when the model was trained.
 
         new_observation_data : SFrame, optional
             ``new_observation_data`` gives additional observation data
@@ -1176,8 +1162,8 @@ class _Recommender(_Model):
         exclude_known : bool, optional
             By default, all user-item interactions previously seen in
             the training data, or in any new data provided using
-            new_observation_data=..., are excluded from the
-            recommendations.  Passing in ``exclude_known = False``
+            new_observation_data.., are excluded from the
+            recommendations.  Passing in ``exclude_known === False``
             overrides this behavior.
 
         verbose : bool, optional
@@ -1202,23 +1188,36 @@ class _Recommender(_Model):
         _mt._get_metric_tracker().track('toolkit.recsys.recommend')
         assert type(k) == int
 
+        __null_sframe = _SFrame()
+
         if users is None:
-            users = _SArray()
+            users = __null_sframe
         if exclude is None:
-            exclude = _SFrame()
+            exclude = __null_sframe
         if items is None:
-            items = _SArray()
+            items = __null_sframe
         if new_observation_data is None:
-            new_observation_data = _SFrame()
+            new_observation_data = __null_sframe
         if new_user_data is None:
-            new_user_data = _SFrame()
+            new_user_data = __null_sframe
         if new_item_data is None:
-            new_item_data = _SFrame()
+            new_item_data = __null_sframe
 
         if isinstance(users, list) or (_HAS_NUMPY and isinstance(users, _numpy.ndarray)):
             users = _SArray(users)
+            # allow to take a list of dictionaries of the form [{'user_id':1,'time':10}] etc.
+            if users.dtype() == dict:
+                users = users.unpack(column_name_prefix='')
+            
+        if isinstance(users, _SArray):
+            users = _SFrame({self.get('user_id'): users})
+            
         if isinstance(items, list) or (_HAS_NUMPY and isinstance(items, _numpy.ndarray)):
             items = _SArray(items)
+            
+        if isinstance(items, _SArray):
+            items = _SFrame({self.get('item_id'): items})
+
 
         def check_type(arg, arg_name, required_type, allowed_types):
             if not isinstance(arg, required_type):
@@ -1226,18 +1225,18 @@ class _Recommender(_Model):
                                 + (", ".join(allowed_types))
                                 + "; Type '" + str(type(arg)) + "' not recognized.")
 
-        check_type(users, "users", _SArray, ["SArray", "list", "numpy.ndarray"])
+        check_type(users, "users", _SFrame, ["SArray", "list", "SFrame", "numpy.ndarray"])
         check_type(exclude, "exclude", _SFrame, ["SFrame"])
-        check_type(items, "items", _SArray, ["SArray", "list", "numpy.ndarray"])
+        check_type(items, "items", _SFrame, ["SFrame", "SArray", "list", "numpy.ndarray"])
         check_type(new_observation_data, "new_observation_data", _SFrame, ["SFrame"])
         check_type(new_user_data, "new_user_data", _SFrame, ["SFrame"])
         check_type(new_item_data, "new_item_data", _SFrame, ["SFrame"])
 
         opt = {'model': self.__proxy__,
-               'users': users,
+               'query': users,
                'top_k': k,
                'exclude': exclude,
-               'items': items,
+               'restrictions': items,
                'new_data': new_observation_data,
                'new_user_data': new_user_data,
                'new_item_data': new_item_data,
