@@ -252,8 +252,12 @@ def create(dataset,
 class TopicModel(_Model):
     """
     TopicModel objects can be used to predict the underlying topic of a
-    document. Use the `create` method in this module to construct a TopicModel
-    instance.
+    document. 
+
+    This model cannot be constructed directly.  Instead, use 
+    :func:`graphlab.topic_model.create` to create an instance
+    of this model. A detailed list of parameter options and code samples 
+    are available in the documentation for the create function.
     """
 
     def __init__(self, model_proxy):
