@@ -14,4 +14,8 @@ class GMT(tzinfo):
         if(self.offset >= 0):
             return "GMT +"+str(self.offset)
         elif(self.offset < 0):
-            return "GMT "+str(self.offset)
\ No newline at end of file
+            return "GMT "+str(self.offset)
+    def __str__(self):
+        return self.tzname(self.offset)
+    def  __repr__(self):
+        return self.tzname(self.offset)
\ No newline at end of file
